/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package dialogs;

import java.util.ResourceBundle;

import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.graphics.ImageData;
import org.eclipse.swt.graphics.PaletteData;
import org.eclipse.swt.graphics.RGB;
import org.eclipse.swt.widgets.Link;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Text;
import org.eclipse.swt.SWT;
import org.eclipse.swt.SWTException;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.program.Program;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;

import core.Loader;

import utils.StockIcon;

/**
 * Instances of this class are used to show
 * a dialog with info about this application,
 * the license it uses and the authors credits
 */
public class AboutDialog {
	
	/**
	 * Instances of this class are used to display
	 * in a new window the application's license
	 */
	private class LicenseDialog {

		/**
		 * The shell used by this dialog
		 */
		private Shell sShell = null;

		/**
		 * This dialog's parent shell
		 */
		private Shell parent = null;

		/**
		 * Text area to display the license as read only
		 */
		private StyledText textArea = null;

		/**
		 * The dialog's close button
		 */
		private Button buttonClose;

		/**
		 * Creates a new instance of this class given it's parent Shell window
		 * @param parent the dialog's parent
		 *
		 * @exception IllegalArgumentException <ul>
		 *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
		 * </ul>
		 * @exception SWTException <ul>
		 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
		 *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
		 * </ul>
		 */
		public LicenseDialog(Shell parent) {
			this.parent = parent;
			createSShell();
		}
		
		/**
		 * This method initializes sShell
		 */
		private void createSShell() {
			
			//grid layout for the shell
			GridLayout gridLayoutSShell = new GridLayout();
			gridLayoutSShell.marginHeight = 12;
			gridLayoutSShell.marginWidth = 12;
			gridLayoutSShell.verticalSpacing = 12;

			//grid data for the text area
			GridData gridDataTextArea = new GridData();
			gridDataTextArea.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
			gridDataTextArea.verticalAlignment = org.eclipse.swt.layout.GridData.FILL;
			gridDataTextArea.grabExcessHorizontalSpace = true;
			gridDataTextArea.grabExcessVerticalSpace = true;
			gridDataTextArea.widthHint = 376;
			gridDataTextArea.heightHint = 246;

			//grid data for the close button
			GridData gridDataButtonClose = new GridData();
			gridDataButtonClose.horizontalAlignment = org.eclipse.swt.layout.GridData.END;
			gridDataButtonClose.verticalAlignment = org.eclipse.swt.layout.GridData.CENTER;
			gridDataButtonClose.grabExcessHorizontalSpace = true;
			
			//initialize the shell
			sShell = new Shell(parent, SWT.DIALOG_TRIM);
			sShell.setText(messages.getString("ABOUT_WINDOW_LICENSE_WINDOW_TITLE"));
			sShell.setLayout(gridLayoutSShell);
			sShell.setImage(AboutDialog.this.parent.getImage());

			//initialize the text area
			textArea = new StyledText(sShell, SWT.MULTI | SWT.WRAP | SWT.V_SCROLL | SWT.BORDER | SWT.READ_ONLY);
			textArea.setLayoutData(gridDataTextArea);
			textArea.setText(messages.getString("ABOUT_WINDOW_LICENCE_WINDOW_TEXT"));
			textArea.setIndent(8);
			
			//initialize the close button
			buttonClose = new Button(sShell, SWT.NONE);
			buttonClose.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_CLOSE, StockIcon.SIZE_LARGE_TOOLBAR, true)));
			buttonClose.setLayoutData(gridDataButtonClose);
			buttonClose.setText(messages.getString("ABOUT_WINDOW_LICENSE_WINDOW_CLOSE_BUTTON"));
			buttonClose.setFocus();
			gridDataButtonClose.widthHint = buttonClose.computeSize(-1, -1).x + 10;
			
			//button listener
			buttonClose.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
				public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
					sShell.close();
				}
			});

			sShell.pack();
			sShell.open();
		}
	}

	/**
	 * The shell used by this dialog
	 */
	private Shell sShell = null;

	/**
	 * Composite to hold the application icon
	 */
	private Composite compositeImage = null;

	/**
	 * The SWT image hold by the image composite
	 */
	private Image appImage;

	/**
	 * Composite used to show the buttons to the bottom of the window
	 */
	private Composite compositeBottom = null;

	//the bottom composite buttons
	private Button buttonCredits = null;
	private Button buttonLicense = null;
	private Button buttonClose = null;

	//text areas to show application information
	//(Text is useful to copy the strings displayed)
	private Text textAppTitle = null;
	private Text textInfo = null;
	private Text textCopyRight1 = null;
	private Text textCopyRight2 = null;
	
	//the link to the application's web site
	private Link applicationLink = null;

	/**
	 * This dialog's parent Shell
	 */
	private final Shell parent;

	/**
	 * messages that are to be used by the entire GUI
	 */
	private ResourceBundle messages;

	/**
	 * Creates a new instance of this class given it's parent Shell window
	 * @param parent the dialog's parent
	 *
	 * @exception IllegalArgumentException <ul>
	 *    <li>ERROR_NULL_ARGUMENT - if the parent is null</li>
	 * </ul>
	 * @exception SWTException <ul>
	 *    <li>ERROR_THREAD_INVALID_ACCESS - if not called from the thread that created the parent</li>
	 *    <li>ERROR_INVALID_SUBCLASS - if this class is not an allowed subclass</li>
	 * </ul>
	 */
	public AboutDialog(Shell parent) {
		this.parent = parent;
		messages = Loader.getMessages();
		createSShell();
	}

	/**
	 * This method initializes sShell
	 */
	private void createSShell() {

		//grid data for the title text
		GridData gridDataTextTitle = new GridData();
		gridDataTextTitle.grabExcessHorizontalSpace = true;
		gridDataTextTitle.horizontalAlignment = GridData.CENTER;
		
		//grid data for the info text
		GridData gridDataTextInfo = new GridData();
		gridDataTextInfo.verticalIndent = 10;

		//grid data for the copyright text 1
		GridData gridDataTextCopyright1 = new GridData();
		gridDataTextCopyright1.grabExcessHorizontalSpace = true;
		gridDataTextCopyright1.horizontalAlignment = GridData.CENTER;

		//grid data for the copyright text 2
		GridData gridDataTextCopyright2 = new GridData();
		gridDataTextCopyright2.grabExcessHorizontalSpace = true;
		gridDataTextCopyright2.horizontalAlignment = GridData.CENTER;

		//grid data for the application link
		GridData gridDataApplicationLink = new GridData();
		gridDataApplicationLink.grabExcessHorizontalSpace = true;
		gridDataApplicationLink.horizontalAlignment = GridData.CENTER;

		//grid layout for the shell
		GridLayout gridLayoutShell = new GridLayout();
		
		//initialize the shell
		sShell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		sShell.setText(messages.getString("ABOUT_WINDOW_TITLE") + " " + System.getProperty("program.name"));
		sShell.setLayout(gridLayoutShell);

		//create the image composite
		createCompositeImage();

		//initialize the application title text
		textAppTitle = new Text(sShell, SWT.READ_ONLY);
		textAppTitle.setFont(new Font(Display.getDefault(), "Sans", 16, SWT.BOLD));
		textAppTitle.setText(System.getProperty("program.name") + " " + System.getProperty("program.version"));
		textAppTitle.setLayoutData(gridDataTextTitle);
		textAppTitle.setBackground(sShell.getBackground());
		
		//initialize the application information text
		textInfo = new Text(sShell, SWT.READ_ONLY);
		textInfo.setText(messages.getString("ABOUT_WINDOW_INFO"));
		textInfo.setBackground(sShell.getBackground());
		
		//initialize the first copyright text
		textCopyRight1 = new Text(sShell, SWT.READ_ONLY);
		textCopyRight1.setFont(new Font(Display.getDefault(), "Sans", 8, SWT.NORMAL));
		textCopyRight1.setLayoutData(gridDataTextCopyright1);
		textCopyRight1.setBackground(sShell.getBackground());
		textCopyRight1.setText(messages.getString("ABOUT_WINDOW_COPYRIGHT1"));
		
		//initialize the second copyright text
		textCopyRight2 = new Text(sShell, SWT.READ_ONLY);
		textCopyRight2.setFont(new Font(Display.getDefault(), "Sans", 8, SWT.NORMAL));
		textCopyRight2.setLayoutData(gridDataTextCopyright2);
		textCopyRight2.setBackground(sShell.getBackground());
		textCopyRight2.setText(messages.getString("ABOUT_WINDOW_COPYRIGHT2"));
		
		//initialize the application link
		applicationLink = new Link(sShell, SWT.NONE);
		applicationLink.setText("<a>" + messages.getString("ABOUT_WINDOW_URL_NAME") + "</a>");
		applicationLink.setLayoutData(gridDataApplicationLink);
		
		//listener for the application link
		applicationLink.addSelectionListener(new SelectionAdapter(){
			public void widgetSelected(SelectionEvent e) {
				Program.launch(messages.getString("ABOUT_WINDOW_URL"));
			}
		});
		
		//initialize the bottom composite with it's controls
		createCompositeBottom();
		
		//pack the shell and keep everything aligned
		sShell.pack();
		
		//the default shell button is the close button
		sShell.setDefaultButton(buttonClose);
		
		//center the shell on screen
		int width = Display.getCurrent().getBounds().width;
		int height = Display.getCurrent().getBounds().height;
		sShell.setLocation(
				(width / 2) - (sShell.getBounds().width / 2),
				(height / 2) - (sShell.getBounds().height / 2)
		);

		//dispose the created image on disposing this dialog
		sShell.addDisposeListener(new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				if (appImage != null) {
					appImage.dispose();
				}
			}
		});
		
		//open the shell
		sShell.open();
	}

	/**
	 * This method initializes compositeImage
	 */
	private void createCompositeImage() {
		
		//image data to hold the application's icon in 64px size
		ImageData imgData = new ImageData(System.getProperty("program.icon"));
		imgData = imgData.scaledTo(64, imgData.height / imgData.width * 64);
		
		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.widthHint = imgData.width;
		gridDataComposite.heightHint = imgData.height;
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = GridData.CENTER;
		gridDataComposite.verticalIndent = 5;
		
		//initialize the composite
		compositeImage = new Composite(sShell, SWT.NONE);
		compositeImage.setLayout(new GridLayout());
		compositeImage.setLayoutData(gridDataComposite);
		
		//draw the image data and try to keep transparency
		PaletteData palette = new PaletteData(new RGB[] { compositeImage.getBackground().getRGB() });
		ImageData id = new ImageData(imgData.width, imgData.height, 1, palette);
		id.transparentPixel = 0;
		appImage = new Image(Display.getCurrent(),id);
		GC g = new GC(appImage);
		g.drawImage(new Image(Display.getCurrent(),imgData), 0, 0);
		g.dispose();
		
		compositeImage.setBackgroundImage(appImage);
	}

	/**
	 * This method initializes compositeBottom
	 */
	private void createCompositeBottom() {
		
		//grid data for the close button
		GridData gridDataButtonClose = new GridData();

		//grid data for the credits button
		GridData gridDataButtonCredits = new GridData();

		//grid data for the license button
		GridData gridDataButtonLicense = new GridData();

		//grid data for the left composite
		GridData gridDataLeftComposite = new GridData();
		gridDataLeftComposite.grabExcessHorizontalSpace = true;
		gridDataLeftComposite.horizontalAlignment = GridData.BEGINNING;
		
		//grid data for the right composite
		GridData gridDataRightComposite = new GridData();
		gridDataRightComposite.grabExcessHorizontalSpace = true;
		gridDataRightComposite.horizontalAlignment = GridData.END;
		
		//grid data for the composite
		GridData gridDataComposite = new GridData();
		gridDataComposite.verticalIndent = 5;
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = GridData.FILL;
		
		//grid layout for the left composite
		GridLayout gridLayoutLeftComposite = new GridLayout();
		gridLayoutLeftComposite.numColumns = 2;

		//grid layout for the right composite
		GridLayout gridLayoutRightComposite = new GridLayout();

		//grid layout for the bottom composite
		GridLayout gridLayoutBottomComposite = new GridLayout();
		gridLayoutBottomComposite.marginHeight = 0;
		gridLayoutBottomComposite.marginWidth = 0;
		gridLayoutBottomComposite.numColumns = 2;

		//initialize the composite
		compositeBottom = new Composite(sShell, SWT.NONE);
		compositeBottom.setLayoutData(gridDataComposite);
		compositeBottom.setLayout(gridLayoutBottomComposite);
		
		//composite to hold the left controls
		Composite leftComposite = new Composite(compositeBottom, SWT.NONE);
		leftComposite.setLayoutData(gridDataLeftComposite);
		leftComposite.setLayout(gridLayoutLeftComposite);
		
		//composite to hold the right controls
		Composite rightComposite = new Composite(compositeBottom, SWT.NONE);
		rightComposite.setLayoutData(gridDataRightComposite);
		rightComposite.setLayout(gridLayoutRightComposite);

		//initialize the credits button in the left composite
		buttonCredits = new Button(leftComposite, SWT.NONE);
		buttonCredits.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_ABOUT, StockIcon.SIZE_LARGE_TOOLBAR, true)));
		buttonCredits.setLayoutData(gridDataButtonCredits);
		buttonCredits.setText(messages.getString("ABOUT_WINDOW_CREDITS_BUTTON"));
		buttonCredits.setEnabled(false);
		
		//initialize the license button in the left composite
		buttonLicense = new Button(leftComposite, SWT.NONE);
		buttonLicense.setLayoutData(gridDataButtonLicense);
		buttonLicense.setText(messages.getString("ABOUT_WINDOW_LICENSE_BUTTON"));
		
		//initialize the close button in the right composite
		buttonClose = new Button(rightComposite, SWT.NONE);
		buttonClose.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_CLOSE, StockIcon.SIZE_LARGE_TOOLBAR, true)));
		buttonClose.setLayoutData(gridDataButtonClose);
		buttonClose.setText(messages.getString("ABOUT_WINDOW_CLOSE_BUTTON"));
		buttonClose.setFocus();
		
		//button listeners
		buttonLicense.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				new LicenseDialog(sShell);
			}
		});
		buttonClose.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				sShell.close();
			}
		});
		
		//adjust all buttons width
		gridDataButtonClose.widthHint = Math.max(buttonClose.computeSize(-1, -1).x + 10, Math.max(buttonCredits.computeSize(-1, -1).x + 10, buttonLicense.computeSize(-1, -1).x + 10));
		gridDataButtonCredits.widthHint = Math.max(buttonClose.computeSize(-1, -1).x + 10, Math.max(buttonCredits.computeSize(-1, -1).x + 10, buttonLicense.computeSize(-1, -1).x + 10));
		gridDataButtonLicense.widthHint = Math.max(buttonClose.computeSize(-1, -1).x + 10, Math.max(buttonCredits.computeSize(-1, -1).x + 10, buttonLicense.computeSize(-1, -1).x + 10));

		//adjust all buttons height
		gridDataButtonClose.heightHint = Math.max(buttonClose.computeSize(-1, -1).y, Math.max(buttonCredits.computeSize(-1, -1).y, buttonLicense.computeSize(-1, -1).y));
		gridDataButtonCredits.heightHint = Math.max(buttonClose.computeSize(-1, -1).y, Math.max(buttonCredits.computeSize(-1, -1).y, buttonLicense.computeSize(-1, -1).y));
		gridDataButtonLicense.heightHint = Math.max(buttonClose.computeSize(-1, -1).y, Math.max(buttonCredits.computeSize(-1, -1).y, buttonLicense.computeSize(-1, -1).y));
	}
}