/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package dialogs;

import java.awt.Color;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.util.ResourceBundle;


import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.GC;
import org.eclipse.swt.widgets.Combo;
import org.eclipse.swt.widgets.Event;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.printing.Printer;
import org.eclipse.swt.printing.PrinterData;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;

import core.Loader;

import utils.ImageUtils;
import utils.StockIcon;

/**
 * Instances of this class allow the user to choose
 * a image for printing with the desired printer
 */
//TODO this class is in early pre-alpha stage,
//more work is needed to make a quality printing.
//all this class does for now is demonstrate that
//printing in SWT is actually possible.
//TODO a better implementation is needed here
public class PrintDialog {

	/**
	 * The shell used by this dialog
	 */
	private Shell sShell = null;

	/**
	 * The composite used to show the image
	 */
	private Composite compositeImage = null;

	/**
	 * The composite used to show the printing controls
	 */
	private Composite compositeControls = null;

	/**
	 * The bottom composite with the print/cancel buttons
	 */
	private Composite compositeBottom = null;

	//bottom composite buttons
	private Button buttonCancel = null;
	private Button buttonPrint = null;

	private Label labelImage = null;
	private Label labelSeparator = null;
	private Combo comboImageChooser = null;

	/**
	 * This dialog's Shell parent
	 */
	private final Shell parent;
	private final BufferedImage beforeImage;
	private final BufferedImage afterImage;
	private final BufferedImage beforeAndAfterImage;
	private final Image swtBeforeImage;
	private final Image swtAfterImage;
	private final Image swtBeforeAndAfterImage;
	private Image swtSelectedImage;
	private ResourceBundle messages;

	//TODO dispose the swt images after use?
	public PrintDialog(Shell parent, BufferedImage beforeImage, BufferedImage afterImage) {
		this.parent = parent;
//		this.beforeImage = new Image(Display.getCurrent(), ImageUtils.convertToSWT(resizedFitImage));
//		this.afterImage = new Image(Display.getCurrent(), ImageUtils.convertToSWT(bufferedImage));

		Graphics2D g2d = null;

		BufferedImage tmpBefore = scaleImage(264, 352, beforeImage);
		BufferedImage tmpAfter =  scaleImage(264, 352, afterImage);

		this.beforeImage = new BufferedImage(264, 407, BufferedImage.TYPE_INT_RGB);
		g2d = this.beforeImage.createGraphics();
		g2d.setBackground(Color.WHITE);
		g2d.fillRect(0, 0, this.beforeImage.getWidth(), this.beforeImage.getHeight());
		g2d.drawImage(tmpBefore,0, (407 - 352) / 2, null);
		g2d.dispose();

		this.afterImage = new BufferedImage(264, 407, BufferedImage.TYPE_INT_RGB);
		g2d = this.afterImage.createGraphics();
		g2d.setBackground(Color.WHITE);
		g2d.fillRect(0, 0, this.afterImage.getWidth(), this.afterImage.getHeight());
		g2d.drawImage(tmpAfter,0, (407 - 352) / 2, null);
		g2d.dispose();

		this.beforeAndAfterImage = createBeforeAndAfterImage(tmpBefore, tmpAfter);

		tmpBefore = null;
		tmpAfter = null;


		swtBeforeImage = new Image(Display.getCurrent(), ImageUtils.convertToSWT(addBorder(this.beforeImage)));
		swtAfterImage = new Image(Display.getCurrent(), ImageUtils.convertToSWT(addBorder(this.afterImage)));
		swtBeforeAndAfterImage = new Image(Display.getCurrent(), ImageUtils.convertToSWT(addBorder(this.beforeAndAfterImage)));

		swtSelectedImage = swtAfterImage;

		messages = Loader.getMessages();
		createSShell();
	}

	private BufferedImage addBorder(BufferedImage image) {
		BufferedImage finalImafe = new BufferedImage(284, 427, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = finalImafe.createGraphics();
		g2d.setBackground(Color.WHITE);
		g2d.fillRect(0, 0, finalImafe.getWidth(), finalImafe.getHeight());
		g2d.drawImage(image, 10, 10, null);
		g2d.dispose();
		return finalImafe;
	}

	private BufferedImage scaleImage(int width, int height, BufferedImage image) {
		java.awt.Image tmp = image.getScaledInstance(width, height, java.awt.Image.SCALE_SMOOTH);
		BufferedImage finalImafe = new BufferedImage(width, height, BufferedImage.TYPE_INT_RGB);
		Graphics2D g2d = finalImafe.createGraphics();
		g2d.drawImage(tmp, 0, 0, null);
		g2d.dispose();
		tmp = null;
		return finalImafe;
	}

	private BufferedImage createBeforeAndAfterImage(BufferedImage tmpBefore, BufferedImage tmpAfter) {

		BufferedImage scaledBefore = scaleImage(198, 264, tmpBefore);
		BufferedImage scaledAfter = scaleImage(198, 264, tmpAfter);

		Graphics2D g2d = null;
		BufferedImage afterFlip = new BufferedImage(264, 198, BufferedImage.TYPE_INT_RGB);
		for(int j=0; j<264; j++)
			for(int i=0; i<198; i++)
				afterFlip.setRGB(j, i, scaledAfter.getRGB(197 - i, j));
		BufferedImage beforeFlip = new BufferedImage(264, 198, BufferedImage.TYPE_INT_RGB);
		for(int j=0; j<264; j++)
			for(int i=0; i<198; i++)
				beforeFlip.setRGB(j, i, scaledBefore.getRGB(197 - i, j));

		scaledBefore = null;
		scaledAfter = null;

		BufferedImage beforeAndAfter =  new BufferedImage(264, 407, BufferedImage.TYPE_INT_RGB);
		g2d = beforeAndAfter.createGraphics();
		g2d.setBackground(Color.WHITE);
		g2d.fillRect(0, 0, beforeAndAfter.getWidth(), beforeAndAfter.getHeight());
		g2d.drawImage(afterFlip, 0, 0, null);
		g2d.drawImage(beforeFlip, 0, 407 - 198, null);
		return beforeAndAfter;
	}

	/**
	 * This method initializes sShell
	 */
	private void createSShell() {
		GridData gridDataLabelSeparator = new GridData();
		gridDataLabelSeparator.grabExcessHorizontalSpace = true;
		gridDataLabelSeparator.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		GridLayout gridLayoutShell = new GridLayout();
		gridLayoutShell.marginTop = 5;
		sShell = new Shell(parent, SWT.APPLICATION_MODAL | SWT.DIALOG_TRIM);
		sShell.setText(messages.getString("PRINT_DIALOG_WINDOW_TITLE"));
		labelImage = new Label(sShell, SWT.NONE);
		labelImage.setText(messages.getString("PRINT_DIALOG_WINDOW_IMAGE_LABEL"));
		sShell.setLayout(gridLayoutShell);
		createCompositeImage();
		createCompositeControls();
		labelSeparator = new Label(sShell, SWT.SEPARATOR | SWT.HORIZONTAL);
		labelSeparator.setLayoutData(gridDataLabelSeparator);

		createCompositeBottom();
		sShell.pack();
		sShell.open();
	}

	/**
	 * This method initializes compositeImage
	 */
	private void createCompositeImage() {
		GridData gridDataComposite = new GridData();
		//TODO -2 values are because SWT.BORDER adds those pixels in composite. why!?
		gridDataComposite.widthHint = swtSelectedImage.getBounds().width - 2;
		gridDataComposite.heightHint = swtSelectedImage.getBounds().height - 2;
		compositeImage = new Composite(sShell, SWT.BORDER);
		compositeImage.setLayout(new GridLayout());
		compositeImage.setLayoutData(gridDataComposite);
		compositeImage.setBackgroundImage(swtSelectedImage);
	}

	/**
	 * This method initializes compositeControls
	 */
	private void createCompositeControls() {
		GridLayout gridLayoutComposite = new GridLayout();
//		gridLayoutComposite.numColumns = 2;
		GridData gridDataComposite = new GridData();
		gridDataComposite.grabExcessHorizontalSpace = true;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		GridData gridDataCombo = new GridData();
		gridDataCombo.grabExcessHorizontalSpace = true;
		gridDataCombo.horizontalAlignment = org.eclipse.swt.layout.GridData.FILL;
		compositeControls = new Composite(sShell, SWT.NONE);
		compositeControls.setLayout(new GridLayout());
		compositeControls.setLayout(gridLayoutComposite);
		compositeControls.setLayoutData(gridDataComposite);
		labelImage = new Label(compositeControls, SWT.NONE);
		labelImage.setText(messages.getString("PRINT_DIALOG_WINDOW_COMBO_LABEL"));
		comboImageChooser = new Combo(compositeControls, SWT.READ_ONLY);
		comboImageChooser.setLayoutData(gridDataCombo);
		comboImageChooser.add(messages.getString("PRINT_DIALOG_WINDOW_COMBO_BEFORE_IMAGE"));
		comboImageChooser.add(messages.getString("PRINT_DIALOG_WINDOW_COMBO_AFTER_IMAGE"));
		comboImageChooser.add(messages.getString("PRINT_DIALOG_WINDOW_COMBO_BEFORE_AND_AFTER_IMAGE"));
		comboImageChooser.addSelectionListener(new SelectionAdapter() {
			public void widgetSelected(SelectionEvent e) {
				if (e.widget != comboImageChooser) return ;
				changePrintImage();
			}
		});
		comboImageChooser.select(1);
		Event e = new Event();
		e.widget = comboImageChooser;
		comboImageChooser.notifyListeners(SWT.Selection, e);

	}

	private void changePrintImage() {
		switch (comboImageChooser.getSelectionIndex()) {
		case 0:
			swtSelectedImage = swtBeforeImage;
			break;
		case 1:
			swtSelectedImage = swtAfterImage;
			break;
		case 2:
			swtSelectedImage = swtBeforeAndAfterImage;
			break;
		default:
			break;
		}
		compositeImage.setBackgroundImage(swtSelectedImage);
	}

	/**
	 * This method initializes compositeBottom
	 */
	private void createCompositeBottom() {
		GridData gridDataButtonCancel = new GridData();
		GridData gridDataButtonPrint = new GridData();
		GridLayout gridLayoutComposite = new GridLayout();
		gridLayoutComposite.marginHeight = 1;
		gridLayoutComposite.marginWidth = 1;
		gridLayoutComposite.numColumns = 2;
		GridData gridDataComposite = new GridData();
		gridDataComposite.horizontalSpan = 2;
		gridDataComposite.horizontalAlignment = org.eclipse.swt.layout.GridData.END;
		compositeBottom = new Composite(sShell, SWT.NONE);
		compositeBottom.setLayoutData(gridDataComposite);
		compositeBottom.setLayout(gridLayoutComposite);
		buttonCancel = new Button(compositeBottom, SWT.NONE);
		buttonCancel.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_CANCEL, StockIcon.SIZE_LARGE_TOOLBAR, true)));
		buttonCancel.setLayoutData(gridDataButtonCancel);
		buttonCancel.setText(messages.getString("PRINT_DIALOG_WINDOW_CANCEL_BUTTON"));
		buttonPrint = new Button(compositeBottom, SWT.NONE);
		buttonPrint.setImage(new Image(Display.getCurrent(), StockIcon.getIcon(StockIcon.ICON_PRINT, StockIcon.SIZE_LARGE_TOOLBAR, true)));
		buttonPrint.setLayoutData(gridDataButtonPrint);
		buttonPrint.setText(messages.getString("PRINT_DIALOG_WINDOW_PRINT_BUTTON"));
//		buttonPrint.setEnabled(false);

		buttonPrint.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				switch (comboImageChooser.getSelectionIndex()) {
				case 0:
					sShell.setVisible(false);
					printImage(beforeImage);
					break;
				case 1:
					sShell.setVisible(false);
					printImage(afterImage);
					break;
				case 2:
					sShell.setVisible(false);
					printImage(beforeAndAfterImage);
					break;

				default:
					break;
				}
			}
		});
		buttonCancel.addSelectionListener(new org.eclipse.swt.events.SelectionAdapter() {
			public void widgetSelected(org.eclipse.swt.events.SelectionEvent e) {
				sShell.close();
			}
		});
		gridDataButtonCancel.widthHint = Math.max(buttonCancel.computeSize(-1, -1).x, buttonPrint.computeSize(-1, -1).x) + 10;
		gridDataButtonPrint.widthHint = Math.max(buttonCancel.computeSize(-1, -1).x, buttonPrint.computeSize(-1, -1).x) + 10;
		gridDataButtonCancel.heightHint = Math.max(buttonCancel.computeSize(-1, -1).y, buttonPrint.computeSize(-1, -1).y);
		gridDataButtonPrint.heightHint = Math.max(buttonCancel.computeSize(-1, -1).y, buttonPrint.computeSize(-1, -1).y);
	}

	protected void printImage(BufferedImage image) {
		org.eclipse.swt.printing.PrintDialog dialog = new org.eclipse.swt.printing.PrintDialog(sShell, SWT.NONE);
		dialog.setText(messages.getString("PRINT_DIALOG_WINDOW_SELECT_PRINTER"));
		PrinterData data = dialog.open();
		final Printer printer = new Printer(data);
		final String jobTitle = comboImageChooser.getItem(comboImageChooser.getSelectionIndex());
		final Image printedImage = new Image(Display.getCurrent(), ImageUtils.convertToSWT(image));
		Thread printingThread = new Thread("Printing") {
			public void run() {
				printer.startJob(jobTitle);
				GC gc = new GC(printer);
				printer.startPage();
				gc.drawImage(printedImage, 14, 420);
				printer.endPage();
				printer.endJob();
				printer.dispose();
				gc.dispose();
				printedImage.dispose();
				sShell.getDisplay().syncExec(new Runnable() {
					public void run() {
						sShell.close();
					}
				});
			}
		};
		printingThread.start();
	}
}