/*******************************************************************************
 * 
 * jKiwi -- A virtual makeover and hairstyler application
 * Copyright (c) 2007-2008 Dan Mihai Ile, Maria Joao Roque Barbado Leal
 * 
 * This file is part of jKiwi.
 * 
 * jKiwi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 * 
 * jKiwi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * 
 *******************************************************************************/

package tabs.hair;

import org.eclipse.swt.graphics.Image;

/**
 * Instances of this class contains a small preview image
 * with the hair color applied to this image and the hair
 * color data used for creating the preview image
 */
public class HairColoredSample {

	/**
	 * The hair color used for the preview image
	 */
	private final HairColorData hairColor;

	/**
	 * The preview image
	 */
	private final Image hairColorImage;

	/**
	 * Creates a new instance of this class given it's hair color data and the preview image
	 * that was created using the given hair color data
	 * @param hairColor the hair color data used for the preview image
	 * @param hairColorImage the preview image
	 */
	public HairColoredSample(HairColorData hairColor, Image hairColorImage) {
		this.hairColor = hairColor;
		this.hairColorImage = hairColorImage;
	}

	/**
	 * The hair color data used for creating the preview image
	 * @return the hair color
	 */
	public HairColorData getHairColor() {
		return hairColor;
	}

	/**
	 * The sample preview image created with this instance's hair color
	 * @return a sample image
	 */
	public Image getHairColorImage() {
		return hairColorImage;
	}
}